@@ -55,7 +55,7 @@ require '../Template.pm';
 my %options = (
                'no cache' => {},
                'simple cache' => { cache => 1 },
-#               'shared cache' => { shared_cache => 1, cache => 1 },
+               'shared cache' => { shared_cache => 1, cache => 1 },
 #               'simple cache, no_includes' => { cache => 1, no_includes => 1},
 #               'blind cache' => { blind_cache => 1},
               );
